@@ -3,20 +3,28 @@
 #define SH_PATH "/bin/sh"
 
 #ifdef DJGPP
-#define BIT_BUCKET "nul"
-#define OP_BINARY O_BINARY
+#  define BIT_BUCKET "nul"
+#  define OP_BINARY O_BINARY
 void Perl_DJGPP_init();
-#define PERL_SYS_INIT(argcp, argvp) STMT_START {        \
+#  define PERL_SYS_INIT(argcp, argvp) STMT_START {        \
     Perl_DJGPP_init();    } STMT_END
-#else
-#define PERL_SYS_INIT(c,v)
-#define BIT_BUCKET "\dev\nul"
-#endif
+#else	/* DJGPP */
+#  define PERL_SYS_INIT(c,v)
+#  ifdef WIN32
+#    define BIT_BUCKET "nul"
+#  else
+#    define BIT_BUCKET "\\dev\\nul" /* "wanna be like, umm, Newlined, or somethin?" */
+#  endif
+#endif	/* DJGPP */
 
 #define PERL_SYS_TERM()
 #define dXSUB_SYS int dummy
 #define TMPPATH "plXXXXXX"
 
+#ifdef WIN32
+#define HAS_UTIME
+#endif
+
 /*
  * 5.003_07 and earlier keyed on #ifdef MSDOS for determining if we were 
  * running on DOS, *and* if we had to cope with 16 bit memory addressing 
@@ -75,6 +83,18 @@ void Perl_DJGPP_init();
  */
 #define fwrite1 fwrite
 
-#define Stat(fname,bufptr) stat((fname),(bufptr))
 #define Fstat(fd,bufptr)   fstat((fd),(bufptr))
 #define Fflush(fp)         fflush(fp)
+
+#ifndef WIN32
+#  define Stat(fname,bufptr) stat((fname),(bufptr))
+#else
+#  define Stat(fname,bufptr) win32_stat((fname),(bufptr))
+#  define my_getenv(var)  getenv(var)
+/*
+ * the following are standard library calls (stdio in particular)
+ * that is being redirected to the perl DLL. This is needed for 
+ * Dynaloading any modules that called stdio functions
+ */
+#  include <win32iop.h>
+#endif	/* WIN32 */
