@@ -22,15 +22,16 @@ BEGIN {
     );
 }
 
+sub opset (;@);
+sub opset_to_hex ($);
+sub opdump (;$);
 use subs @EXPORT_OK;
 
 bootstrap Opcode $VERSION;
 
 _init_optags();
 
-
-*ops_to_opset = \&opset;	# alias for old name
-
+sub ops_to_opset { opset @_ }	# alias for old name
 
 sub opset_to_hex ($) {
     return "(invalid opset)" unless verify_opset($_[0]);
@@ -53,6 +54,7 @@ sub _init_optags {
     my(%all, %seen);
     @all{opset_to_ops(full_opset)} = (); # keys only
 
+    local($_);
     local($/) = "\n=cut"; # skip to optags definition section
     <DATA>;
     $/ = "\n=";		# now read in 'pod section' chunks
@@ -393,8 +395,6 @@ These are a hotchpotch of opcodes still waiting to be considered
 
     bless -- could be used to change ownership of objects (reblessing)
 
-    glob
-
     pushre regcmaybe regcomp subst substcont
 
     sprintf prtf -- can core dump
@@ -495,6 +495,8 @@ information about your system but not be able to change it.
 
     wait waitpid
 
+    glob -- access to Cshell via <`rm *`>
+
 =item :ownprocess
 
     exec exit kill
@@ -558,7 +560,7 @@ Originally designed and implemented by Malcolm Beattie,
 mbeattie@sable.ox.ac.uk as part of Safe version 1.
 
 Split out from Safe module version 1, named opcode tags and other
-changes added by Tim Bunce <Tim.Bunce@ig.co.uk>.
+changes added by Tim Bunce E<lt>F<Tim.Bunce@ig.co.uk>E<gt>.
 
 =cut
 
